package stemmer;


import java.util.*;
import java.io.*;
import java.net.*;
import java.beans.*;
import java.lang.reflect.*;
import junit.framework.*;

import gate.*;
import gate.util.*;
import gate.corpora.TestDocument;
import gate.TestGate;
import gate.gui.MainFrame;
import gate.creole.tokeniser.DefaultTokeniser;
import stemmer.snowball.SnowballProgram;


public class TestSnowballStemmer extends TestCase {

  public TestSnowballStemmer(String name) {
    super(name);
  }

 /** Fixture set up */
  public void setUp() throws Exception {
  }

  public void tearDown() throws Exception {
  } // tearDown

  /** Test the default stemmer */
  public void testSnowballStemmer() throws Exception {
    //get a document
   // System.out.println (TestDocument.getTestServerName() + "tests/doc0.html");
  Document doc = Factory.newDocument( new URL("http://gate.ac.uk/tests/doc0.html") );

    //need to tokenise the doc first
    FeatureMap params = Factory.newFeatureMap();
    params.put(DefaultTokeniser.DEF_TOK_TOKRULES_URL_PARAMETER_NAME,
              "gate:/creole/tokeniser/DefaultTokeniser.rules");
    params.put(DefaultTokeniser.DEF_TOK_GRAMRULES_URL_PARAMETER_NAME,
              "gate:/creole/tokeniser/postprocess.jape");

    params.put(DefaultTokeniser.DEF_TOK_ENCODING_PARAMETER_NAME, "UTF-8");
    DefaultTokeniser tokeniser = (DefaultTokeniser) Factory.createResource(
                          "gate.creole.tokeniser.DefaultTokeniser", params);

    tokeniser.setDocument(doc);
    tokeniser.setAnnotationSetName("TokeniserAS");
    tokeniser.execute();

    //create a default stemmer

    //cannot use Factory, as need to load up the creole definition for the
    //stemmer which is too complex to be worth it for a test
    SnowballStemmer stemmer = new SnowballStemmer();
    stemmer.setLanguage("english");

    stemmer.setDocument(doc);
    stemmer.setAnnotationSetName("TokeniserAS");
    stemmer.setAnnotationType("Token");
    stemmer.setAnnotationFeature("string");
    stemmer.init();
    stemmer.execute();
    assertTrue(! doc.getAnnotations("TokeniserAS").isEmpty());

    // test hardcoded example
    Object [] emptyArgs = new Object[0];
    Class stemClass = Class.forName("stemmer.snowball.ext." + "english" + "Stemmer");
    SnowballProgram stem =  (SnowballProgram)stemClass.newInstance();
    Method stemMethod = stemClass.getMethod("stem", null);
    String tokenString = "defaults";

    stem.setCurrent(tokenString.toLowerCase());
    stemMethod.invoke(stem, emptyArgs);
    assertTrue("default".equals((String)stem.getCurrent()));
    stem.setCurrent("testing");
    stemMethod.invoke(stem, emptyArgs);
     if ( ! stem.getCurrent().equals("test"))
      Err.println("english: expected stem \"test\" and got stem: " + stem.getCurrent());


    stemClass = Class.forName("stemmer.snowball.ext." + "german" + "Stemmer");
    stem =  (SnowballProgram)stemClass.newInstance();
    stemMethod = stemClass.getMethod("stem", null);
    tokenString = "Unterseebooten";
    stem.setCurrent(tokenString.toLowerCase());
    stemMethod.invoke(stem, emptyArgs);
    if ( ! stem.getCurrent().equals("unterseeboot"))
      Err.println("german: expected stem \"unterseeboot\" and got stem: " + stem.getCurrent());


  }

  /** Test suite routine for the test runner */
  public static Test suite() {

    return new TestSuite(TestSnowballStemmer.class);
  } // suite

  public static void main(String[] args) {
    try{
      Gate.init();

      TestSuite suite = new TestSuite();
      suite.addTest(TestSnowballStemmer.suite());

      String junitArgs[] = new String[2];
      junitArgs[0] = "TestSnowballStemmer";
      junitArgs[1] = "";

      junit.swingui.TestRunner.main(junitArgs);

    }catch(Exception e){
      e.printStackTrace();
    }
  }
}